package org.aleabodo.fireworksmc.Patterns;

import java.util.PriorityQueue;

import org.aleabodo.fireworksmc.Fireworks_Mc;
import org.aleabodo.fireworksmc.Tempo;
import org.bukkit.FireworkEffect;
import org.bukkit.Location;
import org.bukkit.scheduler.BukkitRunnable;

public abstract class Pattern {
    protected int duration; // Delay of pattern in ticks
    protected long seed;
    protected int bpm;
    protected Tempo tempo;
    protected int startDelay; // Start delay in ticks until the first rocket detonates
    private boolean cancel = false;
    protected PriorityQueue<ScheduledThing> scheduledThings = new PriorityQueue<>();

    Pattern(long seed, int duration, int bpm, Tempo tempo) {
        this.seed = seed;
        this.duration = duration;
        this.bpm = bpm;
        this.tempo = tempo;
        
        this.generate();
        this.startDelay = scheduledThings.peek().getScheduleTime();
    }

    /**
     * Start pattern
     */
    public void start() {
        if (scheduledThings.isEmpty()) {
            return;
        }
                
        new BukkitRunnable() {
            int tick = scheduledThings.peek().getScheduleTime(); //Zecke

            @Override
            public void run() {
                if (cancel) {
                    this.cancel();
                    return;
                }

                while (scheduledThings.peek().getScheduleTime() == tick) {
                    scheduledThings.poll().schedule();
                    if (scheduledThings.isEmpty()) {
                        this.cancel();
                        return;
                    }
                }

                tick++;
            }
        }.runTaskTimer(Fireworks_Mc.getPlugin(), 0, 1);
    }

    /**
     * Stop/cancel execution of pattern
     */
    public void stop() {
        this.cancel = true;
    }

    /**
     * Generate pattern
     */
    protected abstract void generate();

    /**
     * Schedule rocket
     */
    protected void scheduleRocket(int explosionTime, int flightDuration, Location location, FireworkEffect effect) {
        //scheduledThings.add();
        //TODO
    }

    /*
     * ################### GETTER ###################
     */

    public int getDuration() {
        return this.duration;
    }

    public int getStartDelay() {
        return this.duration;
    }

    /*
     * ################### SETTER ###################
     */

    /**
     * Pattern duration in ticks
     */
    void setDuration(int duration) {
        this.duration = duration;
    }

    /**
     * Pattern bpm in ticks
     */
    void setBpm(int bpm) {
        this.bpm = bpm;
    }
}